@@ -1,8 +1,8 @@
 /*
 	File:		AppleUSBProKbd.c
 	Contains:	Driver for second Apple Pro USB Keyboard interface (multimedia keys)
-	Version:	1.8.1
-	Copyright:	 2000-2001 by Apple, all rights reserved.
+	Version:	2.0.0d1
+	Copyright:	 2000-2002 by Apple, all rights reserved.
 
 	File Ownership:
 
@@ -15,7 +15,7 @@
 		(JG)	Jason Giles
 
 	Change History (most recent first):
-
+		 <5>     3/14/02	AW		Change version number to conform to new standard
 		 <4>	03/13/01	JG		Don't auto-repeat on the mute or eject keys. Fix incorrect plist
 									info (matching to proper version of USB, made entries that
 									somehow became reals into integers).
@@ -37,23 +37,21 @@
 
 #include "AppleUSBProKbd.h"
 
-//#include <libkern/OSByteOrder.h>
-
 #include <IOKit/IOBufferMemoryDescriptor.h>
 #include <IOKit/IOService.h>
-#include <IOKit/IOMessage.h>
 
 #include <IOKit/hidsystem/ev_keymap.h>
 #include <IOKit/hidsystem/IOHIDDescriptorParser.h>
 #include <IOKit/hidsystem/IOHIKeyboard.h>
 #include <IOKit/hidsystem/IOHIDShared.h>
-//#include <IOKit/hidsystem/IOHIDSystem.h>
+#include <IOKit/hidsystem/IOHIDSystem.h>
 
 #include <IOKit/usb/IOUSBInterface.h>
 #include <IOKit/usb/IOUSBPipe.h>
 #include <IOKit/usb/USB.h>
-#include <IOKit/usb/IOUSBLog.h>
+#include <IOKit/IOMessage.h>
 
+#include <libkern/OSByteOrder.h>
 
 //====================================================================================================
 // Defines
@@ -72,73 +70,52 @@ OSDefineMetaClassAndStructors( AppleUSBProKbd, IOHIKeyboard )
 #pragma mark -
 #pragma mark  inherited 
 
-bool
-AppleUSBProKbd::init(OSDictionary *properties)
-{
-  if (!super::init(properties))  return false;
-  
-    mSoundUpIsPressed = false;
-    mSoundDownIsPressed = false;
-    mOutstandingIO = 0;
-    mNeedToClose = false;
-
-    return true;
-}
-
-
-
 //====================================================================================================
 // start
 //====================================================================================================
 
 bool AppleUSBProKbd::start( IOService * provider )
 {
-    IOReturn			err;
-    IOWorkLoop			*wl;
+    IOReturn			err = kIOReturnSuccess;
+
+	// Reset key state flags.
+
+	mSoundUpIsPressed		= FALSE;
+	mSoundDownIsPressed		= FALSE;
+	
+	// To start, call our super's start.
+	
+        if( !super::start( provider ) ) return( false );
 		
+	// Remember my interface info.
 	
-    USBLog(3, "%s[%p]::start - beginning - retain count = %d", getName(), this, getRetainCount());
-    mInterface = OSDynamicCast(IOUSBInterface, provider);
-    if (!mInterface)
-	return false;
-		    
-    if( mInterface->open( this ) == false )
-    {
-	USBError(1, "%s[%p]::start - unable to open provider. returning false", getName(), this);
-	return false;
-    }
-    
-    do {
-        mGate = IOCommandGate::commandGate(this);
-
-        if(!mGate)
-        {
-            USBError(1, "%s[%p]::start - unable to create command gate", getName(), this);
-            break;
-        }
-
-	wl = getWorkLoop();
-	if (!wl)
+	mInterface			= OSDynamicCast(IOUSBInterface, provider);
+        if (!mInterface)
+            return false;
+	
+        // IOLog( "****AppleUSBProKbd::Pro keyboard product ID = %x\n", mInterface->GetDevice()->GetProductID() );
+	
+	// Open the interface.
+	
+	if( mInterface->open( this ) == false )
 	{
-            USBError(1, "%s[%p]::start - unable to find my workloop", getName(), this);
-            break;
-	}
-	
-        if (wl->addEventSource(mGate) != kIOReturnSuccess)
-        {
-            USBError(1, "%s[%p]::start - unable to add gate to work loop", getName(), this);
-            break;
+            IOLog("****AppleUSBProKbd::start - interface open failed.\n");
+            return false;
         }
-
+        
+	fTerminating = FALSE;
+	// Let's get the pipe on the interface so we can read from it.
+	
 	IOUSBFindEndpointRequest	endpointRequest;
+
 	endpointRequest.type 		= kUSBInterrupt;
 	endpointRequest.direction	= kUSBIn;
+
 	mInterruptPipe = mInterface->FindNextPipe( NULL, &endpointRequest );
-	
 	if( !mInterruptPipe )
 	{
-            USBError(1, "%s[%p]::start - unable to get interrupt pipe", getName(), this);
-	    break;
+    	IOLog( "****AppleUSBProKbd::start - could not get the interrupt pipe.\n" );
+         goto failedExit;
 	}
 	
 	// Check for all the usages we expect to find on this device. If we don't find what we're
@@ -146,17 +123,19 @@ bool AppleUSBProKbd::start( IOService * provider )
 	
 	if( VerifyNewDevice() == false )
 	{
-            USBError(1, "%s[%p]::start - VerifyNewDevice was not successful. Ignoring this device", getName(), this);
-	    break;
-	}
+       // Error!
+       IOLog("****AppleUSBProKbd::start - VerifyNewDevice was not successful. Ignoring this device.\n" );
+       
+         goto failedExit;
+    }
 
 	// Setup the read buffer.
 	
 	mMaxPacketSize = endpointRequest.maxPacketSize;
-	mReadDataBuffer = IOBufferMemoryDescriptor::withCapacity( mMaxPacketSize, kIODirectionIn );
+	mReadDataBuffer = IOBufferMemoryDescriptor::withCapacity( 8, kIODirectionIn );
 		
 	mCompletionRoutine.target = (void *) this;
-	mCompletionRoutine.action = (IOUSBCompletionAction) AppleUSBProKbd::InterruptReadHandlerEntry;
+	mCompletionRoutine.action = (IOUSBCompletionAction) AppleUSBProKbd::ReadHandler;
 	mCompletionRoutine.parameter = (void *) 0;  // not used
 
 	mReadDataBuffer->setLength( mMaxPacketSize );
@@ -164,45 +143,31 @@ bool AppleUSBProKbd::start( IOService * provider )
 	// The way to set us up to recieve reads is to call it directly. Each time our read handler
 	// is called, we'll have to do another to make sure we get the next read.
 
-        IncrementOutstandingIO();
+        retain();
 	if( err = mInterruptPipe->Read( mReadDataBuffer, &mCompletionRoutine ) )
 	{
-	    USBError(1, "%s[%p]::start - err (%x) in interrupt read, retain count %d after release", getName(), this, err, getRetainCount());
-            DecrementOutstandingIO();
-	    break;
+		IOLog("****AppleProUSBbd::start - failed to do a read from the interrupt pipe.\n" );
+                release();
+		goto failedExit;
 	}
 	
-        USBError(1, "%s[%p]::start AppleUSBProKeyboard @ %d (0x%x)", getName(), this, mInterface->GetDevice()->GetAddress(), strtol(mInterface->GetDevice()->getLocation(), (char **)NULL, 16));
+	// Looks like we're OK, so return a success.
+	
+	return ( true );
+	
+failedExit:
 
-	// OK- so this is not totally kosher in the IOKit world. You are supposed to call super::start near the BEGINNING
-	// of your own start method. However, the IOHIKeyboard::start method invokes registerService, which we don't want to
-	// do if we get any error up to this point. So we wait and call IOHIKeyboard::start here.
-	if( !super::start(mInterface))
+	if( mPreparsedReportDescriptorData ) 
 	{
-	    USBError(1, "%s[%p]::start - unable to start superclass. returning false", getName(), this);
-	    break;	// error
-	}
-	    
-        return true;
-
-    } while (false);
-	
-    USBLog(3, "%s[%p]::start aborting.  err = 0x%x", getName(), this, err);
-
-    if( mPreparsedReportDescriptorData ) 
         HIDCloseReportDescriptor( mPreparsedReportDescriptorData );
-	
-    if ( mInterruptPipe )
-    {
-	mInterruptPipe->Abort();
-	mInterruptPipe = NULL;
     }
-
-    // stop will clean up everything else
-    stop( provider );
-    provider->close( this );
 	
-    return false;
+	provider->close( this );
+    stop( provider );
+	
+	// Return that we failed if we get here.
+
+    return( false );
 }
 
 //====================================================================================================
@@ -211,23 +176,14 @@ bool AppleUSBProKbd::start( IOService * provider )
 
 void AppleUSBProKbd::stop( IOService * provider )
 {	
-    if( mPreparsedReportDescriptorData ) 
+	if( mPreparsedReportDescriptorData ) 
+	{
         HIDCloseReportDescriptor( mPreparsedReportDescriptorData );
-		
-    if (mGate)
-    {
-	IOWorkLoop	*wl = getWorkLoop();
-	if (wl)
-	    wl->removeEventSource(mGate);
-	mGate->release();
-	mGate = NULL;
     }
-    
+		
     super::stop( provider );
 }
 
-
-
 //====================================================================================================
 // eventFlags - IOHIKeyboard override. This is necessary because we will need to return the state
 //				of the modifier keys on attached keyboards. If we don't, then we the HIDSystem gets
@@ -468,86 +424,90 @@ bool AppleUSBProKbd::VerifyNewDevice ()
 // ReadHandler
 //====================================================================================================
 
-void 
-AppleUSBProKbd::InterruptReadHandlerEntry(OSObject *target, void *param, IOReturn status, UInt32 bufferSizeRemaining)
-{
-    AppleUSBProKbd *	me = OSDynamicCast(AppleUSBProKbd, target);
-
-    if (!me)
-        return;
-    
-    me->InterruptReadHandler(status, bufferSizeRemaining);
-    me->DecrementOutstandingIO();
-}
-
-
-
-void
-AppleUSBProKbd::InterruptReadHandler(IOReturn status, UInt32 bufferSizeRemaining)
-{	 
-    bool		queueAnother = true;
-    IOReturn		err = kIOReturnSuccess;
-
-    switch ( status )
+void AppleUSBProKbd::ReadHandler( 	OSObject *	inTarget,
+										void * 		inParameter,
+										IOReturn	inStatus,
+										UInt32		inBufferSizeRemaining )
+{	
+	AppleUSBProKbd * obj = (AppleUSBProKbd*) inTarget;
+ 
+	switch ( inStatus )
     {
-        case kIOReturnOverrun:
-            USBLog(3, "%s[%p]::InterruptReadHandler kIOReturnOverrun error", getName(), this);
+        case ( kIOReturnSuccess ):
+		{
+			// We got some good stuff, so jump right to our special
+			// button handler.
+            break;
+		}
+        case ( kIOReturnOverrun):
+		{
             // Not sure what to do with this error.  It means more data
             // came back than the size of a descriptor.  Hmmm.  For now
             // just ignore it and assume the data that did come back is
             // useful.
-	    // INTENTIONAL FALL THROUGH
-        case kIOReturnSuccess:
-	    // We got some good stuff, so jump right to our special
-	    // button handler.
-	    HandleSpecialButtons( (UInt8*) mReadDataBuffer->getBytesNoCopy(),((UInt32)mMaxPacketSize - bufferSizeRemaining) );
+			
+            IOLog("****AppleUSBProKbd: overrun error.  ignoring.\n" );
             break;
-
-        case kIOReturnNotResponding:
-            USBLog(3, "%s[%p]::InterruptReadHandler kIOReturnNotResponding error", getName(), this);
-            if ( isInactive() )
-            {
-                  queueAnother = false;
-            }
-	    // if we are not yet inactive, then we should just try again. if we are really disconnected, then
-	    // we will eventually be terminated and isInactive will return true
-	    break;
-
+		}
+        case ( kIOReturnNotResponding ):
+		{
+            // This probably means the device was unplugged.  Now
+            // we need to close the driver.
+			
+            // IOLog("****AppleUSBProKbd: Device unplugged.  Goodbye\n" );
+            goto errorExit;
+		}
  	case kIOReturnAborted:
 	    // This generally means that we are done, because we were unplugged, but not always
-            if (isInactive())
+            if (obj->fTerminating)
 	    {
-                USBLog(3,"%s[%p]::InterruptReadHandler Read aborted. We are terminating", getName(), this);
-		queueAnother = false;
+		// IOLog("%s: Read aborted. We are terminating\n", obj->getName());
+		goto errorExit;
 	    }
-	    else
-            {
-                USBLog(3,"%s[%p]::InterruptReadHandler Read aborted. Don't know why. Trying again", getName(), this);
-            }
+	    
+	    IOLog("%s: Read aborted. Don't know why. Trying again\n", obj->getName());
+	    goto queueAnother;
 	    break;
 
-        default:
+       default:
             // We should handle other errors more intelligently, but
             // for now just return and assume the error is recoverable.
-            USBLog(3, "%s[%p]::InterruptReadHandler error (0x%x) reading interrupt pipe", getName(), this, status);
-            break;
+			
+            IOLog("****AppleUSBProKbd: error reading interrupt pipe\n" );
+            goto queueAnother;
     }
+	
+	// Handle the data.
+	
+    obj->HandleSpecialButtons( (UInt8*) obj->mReadDataBuffer->getBytesNoCopy (),
+								((UInt32) obj->mMaxPacketSize - inBufferSizeRemaining) );
 
-    if (queueAnother)
+queueAnother:
+
+    // Reset the buffer.
+	
+    obj->mReadDataBuffer->setLength( obj->mMaxPacketSize );
+
+    // Queue up another one before we leave.
+	
+    obj->retain();
+    if( (inStatus = obj->mInterruptPipe->Read( obj->mReadDataBuffer, &obj->mCompletionRoutine )) )
     {
-	// Reset the buffer - i doubt this is really necessary
-	mReadDataBuffer->setLength( mMaxPacketSize );
-
-	// Queue up another one before we leave.
-        IncrementOutstandingIO();
-    	err = mInterruptPipe->Read( mReadDataBuffer, &mCompletionRoutine );
-        if ( err != kIOReturnSuccess)
-        {
-            // This is bad.  We probably shouldn't continue on from here.
-            USBError(1, "%s[%p]::InterruptReadHandler -  immediate error 0x%x queueing read\n", getName(), this, err);
-            DecrementOutstandingIO();
-        }
+        // This is bad.  We probably shouldn't continue on from here.
+		
+        IOLog( "****AppleUSBProKbd: immediate error %d queueing read\n", inStatus );
+        obj->release();
+        goto errorExit;
     }
+
+    obj->release();
+    return;
+
+errorExit:
+
+    obj->mInterface->close( obj );
+    obj->release();
+    return;
 }
 
 //====================================================================================================
@@ -571,8 +531,8 @@ void AppleUSBProKbd::HandleSpecialButtons(	UInt8 *	inBufferData,
 						 inBufferData, bufferSize );
 	if( err )
 	{
-            USBError(1, "%s[%p]::HandleSpecialButtons -  HIDGetButtons failed (0x%x)", getName(), this, err);
-            return;
+		IOLog( "****AppleUSBProKbd::HandleSpecialButtons - HIDGetButtonsfailed. Error = %d.\n", (int) err );
+		return;
 	}
 		
 	// Record current time for the keypress posting.
@@ -623,7 +583,8 @@ void AppleUSBProKbd::HandleSpecialButtons(	UInt8 *	inBufferData,
 					break;
 				}
 				default:
-                                    USBError(1, "%s[%p]::HandleSpecialButtons -  no usage found for report. Usage = %d", getName(), this, usageList[reportIndex].usagePage);
+					IOLog( "****AppleUSBProKbd::HandleSpecialButtons - no usage found for report. Usage = %d\n", 
+							(int) usageList[reportIndex].usagePage );
 					break;
 			}
 		}
@@ -660,6 +621,8 @@ UInt32 AppleUSBProKbd::FindKeyboardsAndGetModifiers()
 	OSDictionary	*matchingDictionary = NULL;
 	IOHIKeyboard	*device 			= NULL;
 	Boolean 		value				= false;
+	OSObject 	*adbProperty;
+	const char 	*adbKey;
 	
 	
 	mEventFlags = 0;
@@ -670,7 +633,7 @@ UInt32 AppleUSBProKbd::FindKeyboardsAndGetModifiers()
 	matchingDictionary = IOService::serviceMatching( "IOHIKeyboard" );
 	if( !matchingDictionary )
 	{
-                USBError(1, "%s[%p]::FindKeyboardsAndGetModifiers -  could not get a matching dictionary", getName(), this);
+		IOLog( "****AppleUSBProKeyboard - could not get a matching dictionary.\n" );
 		goto exit;
 	}
 	
@@ -679,7 +642,7 @@ UInt32 AppleUSBProKbd::FindKeyboardsAndGetModifiers()
 	iterator = IOService::getMatchingServices( matchingDictionary );
 	if( !iterator )
 	{
-                USBError(1, "%s[%p]::FindKeyboardsAndGetModifiers -  getMatchingServices failed", getName(), this);
+		IOLog( "***AppleUSBProKeyboard - getMatchingServices failed.\n" );
 		goto exit;
 	}
 	
@@ -687,6 +650,15 @@ UInt32 AppleUSBProKbd::FindKeyboardsAndGetModifiers()
 	//
 	while( (device = (IOHIKeyboard*) iterator->getNextObject()) )
 	{		
+		//Ignore the eventFlags of non-keyboard ADB devices such as audio buttons
+		adbProperty = device->getProperty("ADB Match");
+		if (adbProperty)
+		{
+		    adbKey = ((OSString *)adbProperty)->getCStringNoCopy();
+		    if( *adbKey != '2' )	//If not a keyboard
+			continue;
+		}
+
 		value = false;
 		
 		// Save the caps lock state. If more than one keyboard has it down, that's fine -- we
@@ -721,118 +693,22 @@ IOReturn AppleUSBProKbd::message( UInt32 type, IOService * provider,  void * arg
 {
     switch ( type )
     {
-	case kIOMessageServiceIsTerminated:
-	    USBLog(3, "%s[%p]::message - kIOMessageServiceIsTerminated - ignoring now", getName(), this);
-	    break;
+		case kIOMessageServiceIsTerminated:
+			// IOLog("%s::message - service is terminated - aborting pipe\n", getName());
+			fTerminating = TRUE;		// this will cause us to close
+			if (mInterruptPipe)
+			mInterruptPipe->Abort();
+			break;
 
-	case kIOMessageServiceIsSuspended:
-	case kIOMessageServiceIsResumed:
-	case kIOMessageServiceIsRequestingClose:
-	case kIOMessageServiceWasClosed: 
-	case kIOMessageServiceBusyStateChange:
-	default:
+		case kIOMessageServiceIsSuspended:
+		case kIOMessageServiceIsResumed:
+		case kIOMessageServiceIsRequestingClose:
+		case kIOMessageServiceWasClosed: 
+		case kIOMessageServiceBusyStateChange:
+		default:
             break;
     }
     
     return kIOReturnSuccess;
 }
 
-
-
-bool
-AppleUSBProKbd::willTerminate( IOService * provider, IOOptionBits options )
-{
-    // this method is intended to be used to stop any pending I/O and to make sure that 
-    // we have begun getting our callbacks in order. by the time we get here, the 
-    // isInactive flag is set, so we really are marked as being done. we will do in here
-    // what we used to do in the message method (this happens first)
-    USBLog(3, "%s[%p]::willTerminate isInactive = %d", getName(), this, isInactive());
-    if (mReadDataBuffer) 
-    {
-	mReadDataBuffer->release();
-        mReadDataBuffer = NULL;
-    }
-    if (mInterruptPipe)
-    {
-	mInterruptPipe->Abort();
-	mInterruptPipe = NULL;
-    }
-    return super::willTerminate(provider, options);
-}
-
-
-bool
-AppleUSBProKbd::didTerminate( IOService * provider, IOOptionBits options, bool * defer )
-{
-    // this method comes at the end of the termination sequence. Hopefully, all of our outstanding IO is complete
-    // in which case we can just close our provider and IOKit will take care of the rest. Otherwise, we need to 
-    // hold on to the device and IOKit will terminate us when we close it later
-    USBLog(3, "%s[%p]::didTerminate isInactive = %d, outstandingIO = %d", getName(), this, isInactive(), mOutstandingIO);
-    if (!mOutstandingIO)
-	mInterface->close(this);
-    else
-	mNeedToClose = true;
-    return super::didTerminate(provider, options, defer);
-}
-
-
-void
-AppleUSBProKbd::DecrementOutstandingIO(void)
-{
-    if (!mGate)
-    {
-	if (!--mOutstandingIO && mNeedToClose)
-	{
-	    USBLog(3, "%s[%p]::DecrementOutstandingIO isInactive = %d, outstandingIO = %d - closing device", getName(), this, isInactive(), mOutstandingIO);
-	    mInterface->close(this);
-	}
-	return;
-    }
-    mGate->runAction(ChangeOutstandingIO, (void*)-1);
-}
-
-
-void
-AppleUSBProKbd::IncrementOutstandingIO(void)
-{
-    if (!mGate)
-    {
-	mOutstandingIO++;
-	return;
-    }
-    mGate->runAction(ChangeOutstandingIO, (void*)1);
-}
-
-
-IOReturn
-AppleUSBProKbd::ChangeOutstandingIO(OSObject *target, void *param1, void *param2, void *param3, void *param4)
-{
-    AppleUSBProKbd *me = OSDynamicCast(AppleUSBProKbd, target);
-    UInt32	direction = (UInt32)param1;
-    
-    if (!me)
-    {
-	USBLog(1, "AppleUSBProKbd::ChangeOutstandingIO - invalid target");
-	return kIOReturnSuccess;
-    }
-    switch (direction)
-    {
-	case 1:
-	    me->mOutstandingIO++;
-	    break;
-	    
-	case -1:
-	    if (!--me->mOutstandingIO && me->mNeedToClose)
-	    {
-		USBLog(3, "%s[%p]::ChangeOutstandingIO isInactive = %d, outstandingIO = %d - closing device", me->getName(), me, me->isInactive(), me->mOutstandingIO);
-		me->mInterface->close(me);
-	    }
-	    break;
-	    
-	default:
-	    USBLog(1, "%s[%p]::ChangeOutstandingIO - invalid direction", me->getName(), me);
-    }
-    return kIOReturnSuccess;
-}
-
-
